@@ -49,10 +49,15 @@
 #include "gdcmASN1.h"
 #include "gdcmAttribute.h"
 #include "gdcmBase64.h"
+#include "gdcmMEC_MR3.h"
 #include "gdcmTagKeywords.h"
 
 #include <string>
 #include <iostream>
+#ifdef GDCM_HAVE_CODECVT
+#include <codecvt>
+#endif
+#include <locale>
 
 #include <stdio.h>     /* for printf */
 #include <stdlib.h>    /* for exit */
@@ -355,6 +360,7 @@ static bool DumpToshibaDTI( const char * input, size_t len )
   ds.Read<gdcm::ExplicitDataElement,gdcm::SwapperNoOp>( is );
 
   gdcm::Printer p;
+  //gdcm::DictPrinter p;
   p.SetFile( file );
   p.SetColor( color != 0 );
   p.Print( std::cout );
@@ -368,7 +374,7 @@ static int DumpTOSHIBA_Reverse(const gdcm::DataSet & ds, const gdcm::PrivateTag
   // (0029,0010) ?? (LO) [PMTF INFORMATION DATA ]                      # 22,1 Private Creator
   // (0029,1001) ?? (SQ) (Sequence with undefined length)              # u/l,1 ?
 
-  if( !ds.FindDataElement( tpmtf) ) return 1;
+  if( !ds.FindDataElement( tpmtf) ) return 0; // this is not an error at this point
   const gdcm::DataElement& pmtf = ds.GetDataElement( tpmtf );
   if ( pmtf.IsEmpty() ) return 1;
   gdcm::SmartPointer<gdcm::SequenceOfItems> seq = pmtf.GetValueAsSQ();
@@ -489,7 +495,7 @@ struct Data2
    os << "  UserAdress4: " << std::string(UserAdress4,32) << "\n";
    os << "  UserAdress5: " << std::string(UserAdress5,32) << "\n";
    os << "  RecDate: "     << std::string(RecDate,8) << "\n";
-   os << "  RecTime: "     << std::string(RecTime,64) << "\n";
+   os << "  RecTime: "     << std::string(RecTime,6) << "\n";
    os << "  RecPlace: "    << std::string(RecPlace,64) << "\n";
    os << "  RecSource: "   << std::string(RecSource,64) << "\n";
    os << "  DF1: "         << std::string(DF1,64) << "\n";
@@ -849,6 +855,38 @@ static int PrintCT3(const std::string & filename, bool verbose)
   return ret;
 }
 
+static int PrintMEC_MR3(const std::string & filename, bool verbose)
+{
+  (void)verbose;
+  gdcm::Reader reader;
+  reader.SetFileName( filename.c_str() );
+  if( !reader.Read() )
+    {
+    std::cerr << "Failed to read: " << filename << std::endl;
+    return 1;
+    }
+
+  const gdcm::DataSet& ds = reader.GetFile().GetDataSet();
+
+  // Original Data
+  const gdcm::PrivateTag tmecmr3(0x700d,0x08,"TOSHIBA_MEC_MR3");
+  if( !ds.FindDataElement( tmecmr3) ) return 1;
+  const gdcm::DataElement& mecmr3 = ds.GetDataElement( tmecmr3 );
+  if ( mecmr3.IsEmpty() ) return 1;
+  const gdcm::ByteValue * bv = mecmr3.GetByteValue();
+
+  const char *inbuffer = bv->GetPointer();
+  const size_t buf_len= bv->GetLength();
+
+  int ret = 0;
+  if (!gdcm::MEC_MR3::Print(inbuffer, buf_len))
+  {
+    ret = 1;
+  }
+
+  return ret;
+}
+
 static int PrintPMTF(const std::string & filename, bool verbose)
 {
   (void)verbose;
@@ -861,14 +899,61 @@ static int PrintPMTF(const std::string & filename, bool verbose)
     }
 
   const gdcm::DataSet& ds = reader.GetFile().GetDataSet();
+  int ret = 0;
+  {
   const gdcm::PrivateTag tpmtf(0x0029,0x1,"PMTF INFORMATION DATA");
-  const gdcm::PrivateTag tseq(0x0029,0x90,"PMTF INFORMATION DATA");
-  int ret = cleanup::DumpTOSHIBA_Reverse( ds, tpmtf, tseq );
+  static const gdcm::PrivateTag &tseq = gdcm::MEC_MR3::GetPMTFInformationDataTag();
+  ret += cleanup::DumpTOSHIBA_Reverse( ds, tpmtf, tseq );
+  }
+
+  {
+  const gdcm::PrivateTag tpmtf(0x0029,0x1,"CANON_MEC_MR3");
+  static const gdcm::PrivateTag &tseq = gdcm::MEC_MR3::GetCanonMECMR3Tag();
+  ret += cleanup::DumpTOSHIBA_Reverse( ds, tpmtf, tseq );
+
+  // PAS Reproduct Information
+  const gdcm::PrivateTag tpasri(0x700d,0x19,"CANON_MEC_MR3^10");
+  if( ds.FindDataElement( tpasri) ) {
+  const gdcm::DataElement& pasri = ds.GetDataElement( tpasri );
+  if (! pasri.IsEmpty() ) {
+  const gdcm::ByteValue * bv = pasri.GetByteValue();
+  std::string s(bv->GetPointer(), bv->GetLength() );
+  std::cout << std::endl;
+  std::cout << "PAS Reproduct Information (XML)" << std::endl;
+  // header states: 
+  // <?xml version="1.0" encoding="UTF-8"?>
+  // so simply dump without further cleanup:
+  std::cout << s.c_str() << std::endl;
+  }
+  }
+  }
+
+  {
+  const gdcm::PrivateTag tpmtf(0x0029,0x1,"TOSHIBA_MEC_MR3");
+  static const gdcm::PrivateTag &tseq = gdcm::MEC_MR3::GetToshibaMECMR3Tag();
+  ret += cleanup::DumpTOSHIBA_Reverse( ds, tpmtf, tseq );
+
+  const gdcm::PrivateTag tpmtf2(0x0029,0x2,"TOSHIBA_MEC_MR3");
+  ret += cleanup::DumpTOSHIBA_Reverse( ds, tpmtf2, tseq );
+  }
 
   return ret;
 }
 
-static int PrintMECMR3(const std::string & filename, bool verbose)
+static void print_utf8(std::string const& s)
+{
+  const char delim = 0;
+  auto start = 0U;
+  auto end = s.find(delim);
+  while (end != std::string::npos)
+  {
+      std::cout << s.substr(start, end - start) << std::endl;
+      start = end + 1;
+      end = s.find(delim, start);
+  }
+}
+
+static int PrintMedComHistory(const std::string & filename, bool verbose)
 {
   (void)verbose;
   gdcm::Reader reader;
@@ -880,18 +965,26 @@ static int PrintMECMR3(const std::string & filename, bool verbose)
     }
 
   const gdcm::DataSet& ds = reader.GetFile().GetDataSet();
-  const gdcm::PrivateTag tpmtf(0x0029,0x1,"TOSHIBA_MEC_MR3");
-  const gdcm::PrivateTag tseq(0x0029,0x90,"TOSHIBA_MEC_MR3");
-  int ret = cleanup::DumpTOSHIBA_Reverse( ds, tpmtf, tseq );
+  const gdcm::PrivateTag tmedcom(0x0029,0x20,"SIEMENS MEDCOM HEADER");
+  if( !ds.FindDataElement( tmedcom) ) return 1;
+  const gdcm::DataElement& medcom = ds.GetDataElement( tmedcom);
+  if ( medcom.IsEmpty() ) return 1;
+  const gdcm::ByteValue * bv = medcom.GetByteValue();
 
-  const gdcm::PrivateTag tpmtf2(0x0029,0x2,"TOSHIBA_MEC_MR3");
-  int ret2 = cleanup::DumpTOSHIBA_Reverse( ds, tpmtf2, tseq );
+  const size_t size = bv->GetLength();
+  std::u16string u16((size / 2) + 0, '\0');
+  bv->GetBuffer( (char*)&u16[0], size );
 
-  return ret +ret2;
+#ifdef GDCM_HAVE_CODECVT
+  std::string utf8 = std::wstring_convert<
+    std::codecvt_utf8_utf16<char16_t>, char16_t>{}.to_bytes(u16);
+  print_utf8(utf8);
+#else
+  std::cerr << "Missing GDCM_HAVE_CODECVT support" << std::endl;
+#endif
+  return 0;
 }
 
-
-
 static int PrintPDB(const std::string & filename, bool verbose)
 {
   (void)verbose;
@@ -1157,6 +1250,7 @@ static void PrintHelp()
   std::cout << "     --csa-asl        print decoded SIEMENS CSA MR_ASL (base64)." << std::endl;
   std::cout << "     --csa-diffusion  print decoded SIEMENS CSA MRDiffusion (base64)." << std::endl;
   std::cout << "     --mrprotocol     print SIEMENS CSA MrProtocol only (within ASCCONV BEGIN/END)." << std::endl;
+  std::cout << "                      or Phoenix Meta Protocol (0021,19,SIEMENS MR SDS 01)." << std::endl;
   std::cout << "  -P --pdb            print GEMS Protocol Data Block (0025,1b,GEMS_SERS_01)." << std::endl;
   std::cout << "     --elscint        print ELSCINT Protocol Information (01f7,26,ELSCINT1)." << std::endl;
   std::cout << "     --vepro          print VEPRO Protocol Information (0055,20,VEPRO VIF 3.0 DATA)." << std::endl;
@@ -1164,7 +1258,10 @@ static void PrintHelp()
   std::cout << "     --sds            print Philips MR Series Data Storage (1.3.46.670589.11.0.0.12.2) Information (2005,32,Philips MR Imaging DD 002)." << std::endl;
   std::cout << "     --ct3            print CT Private Data 2 (7005,10,TOSHIBA_MEC_CT3)." << std::endl;
   std::cout << "     --pmtf           print PMTF INFORMATION DATA sub-sequences (0029,01,PMTF INFORMATION DATA)." << std::endl;
-  std::cout << "     --mecmr3         print TOSHIBA_MEC_MR3 sub-sequences (0029,01,TOSHIBA_MEC_MR3)." << std::endl;
+  std::cout << "                      or TOSHIBA_MEC_MR3 sub-sequences (0029,01,TOSHIBA_MEC_MR3)." << std::endl;
+  std::cout << "                      or CANON_MEC_MR3 sub-sequences (0029,01,CANON_MEC_MR3)." << std::endl;
+  std::cout << "     --medcom         print MedCom History Information as UTF-8 (0029,20,SIEMENS MEDCOM HEADER)." << std::endl;
+  std::cout << "     --mr3            print Original Data as UTF-8 (700d,08,TOSHIBA_MEC_MR3)." << std::endl;
   std::cout << "  -A --asn1           print encapsulated ASN1 structure >(0400,0520)." << std::endl;
   std::cout << "     --map-uid-names  map UID to names." << std::endl;
   std::cout << "General Options:" << std::endl;
@@ -1190,6 +1287,7 @@ int main (int argc, char *argv[])
   int printcsa = 0;
   int printmrprotocol = 0;
   int printcsabase64 = 0;
+  int printmedcom = 0; // MedCom History Information
   int printcsaasl = 0;
   int printcsadiffusion = 0;
   int printpdb = 0;
@@ -1197,8 +1295,8 @@ int main (int argc, char *argv[])
   int printvepro = 0;
   int printsds = 0; // MR Series Data Storage
   int printct3 = 0; // TOSHIBA_MEC_CT3
-  int printpmtf = 0; // TOSHIBA / PMTF INFORMATION DATA
-  int printmecmr3 = 0; // TOSHIBA / TOSHIBA_MEC_MR3
+  int printmecmr3 = 0; // TOSHIBA_MEC_MR3
+  int printpmtf = 0; // TOSHIBA / PMTF INFORMATION DATA & TOSHIBA / TOSHIBA_MEC_MR3 & CANON_MEC_MR3
   int verbose = 0;
   int warning = 0;
   int debug = 0;
@@ -1245,7 +1343,9 @@ int main (int argc, char *argv[])
         {"csa-diffusion", 0, &printcsadiffusion, 1},
         {"mrprotocol", 0, &printmrprotocol, 1},
         {"pmtf", 0, &printpmtf, 1},
-        {"mecmr3", 0, &printmecmr3, 1},
+        {"mecmr3", 0, &printpmtf, 1},
+        {"medcom", 0, &printmedcom, 1},
+        {"mr3", 0, &printmecmr3, 1},
         {nullptr, 0, nullptr, 0} // required
     };
     static const char short_options[] = "i:xrpdcCPAVWDEhvI";
@@ -1469,7 +1569,11 @@ int main (int argc, char *argv[])
         }
       else if( printmecmr3 )
         {
-        res += PrintMECMR3(*it, verbose!= 0);
+        res += PrintMEC_MR3(*it, verbose!=0);
+        }
+      else if( printmedcom )
+        {
+        res += PrintMedComHistory(*it, verbose!=0);
         }
       else if( printelscint )
         {
@@ -1532,7 +1636,11 @@ int main (int argc, char *argv[])
       }
     else if( printmecmr3 )
       {
-      res += PrintMECMR3(filename, verbose!= 0);
+      res += PrintMEC_MR3(filename, verbose!= 0);
+      }
+    else if( printmedcom )
+      {
+      res += PrintMedComHistory(filename, verbose!= 0);
       }
     else if( printelscint )
       {
